@@ -831,10 +831,15 @@ void Client::updateShape()
             noborder = true;
             updateDecoration(true);
         }
-    }
-    if (shape() && noBorder())
-        XShapeCombineShape(display(), frameId(), ShapeBounding,
+        if (noBorder())
+            XShapeCombineShape(display(), frameId(), ShapeBounding,
                            clientPos().x(), clientPos().y(), window(), ShapeBounding, ShapeSet);
+    } else if (app_noborder) {
+        XShapeCombineMask(display(), frameId(), ShapeBounding, 0, 0, None, ShapeSet);
+        detectNoBorder();
+        app_noborder = noborder;
+        updateDecoration(true);
+    }
 
     // Decoration mask (i.e. 'else' here) setting is done in setMask()
     // when the decoration calls it or when the decoration is created/destroyed
@@ -2003,9 +2008,12 @@ void Client::getMotifHints()
 
     // mminimize; - Ignore, bogus - E.g. shading or sending to another desktop is "minimizing" too
     // mmaximize; - Ignore, bogus - Maximizing is basically just resizing
+    const bool closabilityChanged = motif_may_close != mclose;
     motif_may_close = mclose; // Motif apps like to crash when they set this hint and WM closes them anyway
     if (isManaged())
         updateDecoration(true);   // Check if noborder state has changed
+    if (decoration && closabilityChanged)
+        decoration->reset(KDecoration::SettingButtons);
 }
 
 void Client::readIcons(Window win, QPixmap* icon, QPixmap* miniicon, QPixmap* bigicon, QPixmap* hugeicon)
