@@ -23,6 +23,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // tabbox
 #include "clientitemdelegate.h"
 #include "clientmodel.h"
+#include "declarative.h"
 #include "desktopitemdelegate.h"
 #include "desktopmodel.h"
 #include "itemlayoutconfig.h"
@@ -30,6 +31,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "tabboxview.h"
 // Qt
 #include <qdom.h>
+#include <QtCore/QtConcurrentRun>
+#include <QtCore/QFutureWatcher>
 #include <QFile>
 #include <QKeyEvent>
 #include <QModelIndex>
@@ -68,12 +71,16 @@ public:
 
     ClientModel* clientModel() const;
     DesktopModel* desktopModel() const;
+    void createView();
     void parseConfig(const QString& fileName);
 
     TabBoxHandler *q; // public pointer
     // members
     TabBoxConfig config;
     TabBoxView* view;
+    DeclarativeView *m_declarativeView;
+    ClientModel* m_clientModel;
+    DesktopModel* m_desktopModel;
     QModelIndex index;
     /**
     * Indicates if the tabbox is shown.
@@ -85,35 +92,58 @@ public:
 };
 
 TabBoxHandlerPrivate::TabBoxHandlerPrivate(TabBoxHandler *q)
+    : view(NULL)
+    , m_declarativeView(NULL)
 {
     this->q = q;
     isShown = false;
     lastRaisedClient = 0;
     lastRaisedClientSucc = 0;
     config = TabBoxConfig();
-    view = new TabBoxView();
+    m_clientModel = new ClientModel(q);
+    m_desktopModel = new DesktopModel(q);
 
     // load the layouts
-    parseConfig(KStandardDirs::locate("data", "kwin/DefaultTabBoxLayouts.xml"));
-    view->clientDelegate()->setConfig(tabBoxLayouts.value("Default"));
-    view->additionalClientDelegate()->setConfig(tabBoxLayouts.value("Text"));
-    view->desktopDelegate()->setConfig(tabBoxLayouts.value("Desktop"));
-    view->desktopDelegate()->setLayouts(tabBoxLayouts);
+    QFuture< void> future = QtConcurrent::run(this, &TabBoxHandlerPrivate::parseConfig, KStandardDirs::locate("data", "kwin/DefaultTabBoxLayouts.xml"));
+    QFutureWatcher< void > *watcher = new QFutureWatcher< void >(q);
+    watcher->setFuture(future);
+    q->connect(watcher, SIGNAL(finished()), q, SIGNAL(ready()));
 }
 
 TabBoxHandlerPrivate::~TabBoxHandlerPrivate()
 {
     delete view;
+    delete m_declarativeView;
+}
+
+void TabBoxHandlerPrivate::createView()
+{
+    view = new TabBoxView(m_clientModel, m_desktopModel);
+    if (tabBoxLayouts.contains(config.layoutName())) {
+        view->clientDelegate()->setConfig(tabBoxLayouts.value(config.layoutName()));
+        view->desktopDelegate()->setConfig(tabBoxLayouts.value(config.layoutName()));
+    } else {
+        view->clientDelegate()->setConfig(tabBoxLayouts.value("Default"));
+        view->desktopDelegate()->setConfig(tabBoxLayouts.value("Desktop"));
+    }
+    if (tabBoxLayouts.contains(config.selectedItemLayoutName())) {
+        view->additionalClientDelegate()->setConfig(tabBoxLayouts.value(config.selectedItemLayoutName()));
+    } else {
+        view->additionalClientDelegate()->setConfig(tabBoxLayouts.value("Text"));
+    }
+    view->desktopDelegate()->setLayouts(tabBoxLayouts);
+    emit q->configChanged();
+    view->setCurrentIndex(index);
 }
 
 ClientModel* TabBoxHandlerPrivate::clientModel() const
 {
-    return view->clientModel();
+    return m_clientModel;
 }
 
 DesktopModel* TabBoxHandlerPrivate::desktopModel() const
 {
-    return view->desktopModel();
+    return m_desktopModel;
 }
 
 void TabBoxHandlerPrivate::updateOutline()
@@ -121,12 +151,11 @@ void TabBoxHandlerPrivate::updateOutline()
     if (config.tabBoxMode() != TabBoxConfig::ClientTabBox)
         return;
 //     if ( c == NULL || !m_isShown || !c->isShown( true ) || !c->isOnCurrentDesktop())
-    if (!isShown || view->clientModel()->data(index, ClientModel::EmptyRole).toBool()) {
+    if (!isShown || clientModel()->data(index, ClientModel::EmptyRole).toBool()) {
         q->hideOutline();
         return;
     }
-    TabBoxClient* c = static_cast< TabBoxClient* >(
-                          view->clientModel()->data(index, ClientModel::ClientRole).value<void *>());
+    TabBoxClient* c = static_cast< TabBoxClient* >(m_clientModel->data(index, ClientModel::ClientRole).value<void *>());
     q->showOutline(QRect(c->x(), c->y(), c->width(), c->height()));
 }
 
@@ -158,8 +187,14 @@ void TabBoxHandlerPrivate::updateHighlightWindows()
 
     WId wId;
     QVector< WId > data;
-    if (config.isShowTabBox()) {
-        wId = view->winId();
+    QWidget *w = NULL;
+    if (view && view->isVisible()) {
+        w = view;
+    } else if (m_declarativeView && m_declarativeView->isVisible()) {
+        w = m_declarativeView;
+    }
+    if (config.isShowTabBox() && w) {
+        wId = w->winId();
         data.resize(2);
         data[ 1 ] = wId;
     } else {
@@ -188,7 +223,7 @@ void TabBoxHandlerPrivate::endHighlightWindows(bool abort)
     // highlight windows
     Display *dpy = QX11Info::display();
     Atom atom = XInternAtom(dpy, "_KDE_WINDOW_HIGHLIGHT", False);
-    XDeleteProperty(dpy, config.isShowTabBox() ? view->winId() : QX11Info::appRootWindow(), atom);
+    XDeleteProperty(dpy, config.isShowTabBox() && m_declarativeView ? m_declarativeView->winId() : QX11Info::appRootWindow(), atom);
 }
 
 /***********************************************************
@@ -352,14 +387,14 @@ void TabBoxHandler::setConfig(const TabBoxConfig& config)
 {
     if (config.layoutName() != d->config.layoutName()) {
         // new item layout config
-        if (d->tabBoxLayouts.contains(config.layoutName())) {
+        if (d->tabBoxLayouts.contains(config.layoutName()) && d->view) {
             d->view->clientDelegate()->setConfig(d->tabBoxLayouts.value(config.layoutName()));
             d->view->desktopDelegate()->setConfig(d->tabBoxLayouts.value(config.layoutName()));
         }
     }
     if (config.selectedItemLayoutName() != d->config.selectedItemLayoutName()) {
         // TODO: desktop layouts
-        if (d->tabBoxLayouts.contains(config.selectedItemLayoutName()))
+        if (d->tabBoxLayouts.contains(config.selectedItemLayoutName()) && d->view)
             d->view->additionalClientDelegate()->setConfig(d->tabBoxLayouts.value(config.selectedItemLayoutName()));
     }
     d->config = config;
@@ -376,8 +411,20 @@ void TabBoxHandler::show()
         d->updateOutline();
     }
     if (d->config.isShowTabBox()) {
-        d->view->show();
-        d->view->updateGeometry();
+        if (d->config.tabBoxMode() == TabBoxConfig::ClientTabBox) {
+            // use declarative view
+            if (!d->m_declarativeView) {
+                d->m_declarativeView = new DeclarativeView(d->clientModel());
+            }
+            d->m_declarativeView->show();
+            d->m_declarativeView->setCurrentIndex(d->index);
+        } else {
+            if (!d->view) {
+                d->createView();
+            }
+            d->view->show();
+            d->view->updateGeometry();
+        }
     }
     if (d->config.isHighlightWindows()) {
         d->updateHighlightWindows();
@@ -393,7 +440,12 @@ void TabBoxHandler::hide(bool abort)
     if (d->config.isShowOutline()) {
         hideOutline();
     }
-    d->view->hide();
+    if (d->view) {
+        d->view->hide();
+    }
+    if (d->m_declarativeView) {
+        d->m_declarativeView->hide();
+    }
 }
 
 QModelIndex TabBoxHandler::nextPrev(bool forward) const
@@ -479,7 +531,18 @@ int TabBoxHandler::currentSelectedDesktop() const
 
 void TabBoxHandler::setCurrentIndex(const QModelIndex& index)
 {
-    d->view->setCurrentIndex(index);
+    if (d->index == index) {
+        return;
+    }
+    if (!index.isValid()) {
+        return;
+    }
+    if (d->view) {
+        d->view->setCurrentIndex(index);
+    }
+    if (d->m_declarativeView) {
+        d->m_declarativeView->setCurrentIndex(index);
+    }
     d->index = index;
     if (d->config.tabBoxMode() == TabBoxConfig::ClientTabBox) {
         if (d->config.isShowOutline()) {
@@ -491,6 +554,11 @@ void TabBoxHandler::setCurrentIndex(const QModelIndex& index)
     }
 }
 
+const QModelIndex& TabBoxHandler::currentIndex() const
+{
+    return d->index;
+}
+
 QModelIndex TabBoxHandler::grabbedKeyEvent(QKeyEvent* event) const
 {
     QModelIndex ret;
@@ -542,14 +610,27 @@ QModelIndex TabBoxHandler::grabbedKeyEvent(QKeyEvent* event) const
 
 bool TabBoxHandler::containsPos(const QPoint& pos) const
 {
-    return d->view->geometry().contains(pos);
+    QWidget *w = NULL;
+    if (d->view && d->view->isVisible()) {
+        w = d->view;
+    } else if (d->m_declarativeView && d->m_declarativeView->isVisible()) {
+        w = d->m_declarativeView;
+    } else {
+        return false;
+    }
+    return w->geometry().contains(pos);
 }
 
 QModelIndex TabBoxHandler::indexAt(const QPoint& pos) const
 {
-    QPoint widgetPos = d->view->mapFromGlobal(pos);
-    QModelIndex ret = d->view->indexAt(widgetPos);
-    return ret;
+    if (d->view && d->view->isVisible()) {
+        QPoint widgetPos = d->view->mapFromGlobal(pos);
+        return d->view->indexAt(widgetPos);
+    } else if (d->m_declarativeView && d->m_declarativeView->isVisible()) {
+        QPoint widgetPos = d->m_declarativeView->mapFromGlobal(pos);
+        return d->m_declarativeView->indexAt(widgetPos);
+    }
+    return QModelIndex();
 }
 
 QModelIndex TabBoxHandler::index(KWin::TabBox::TabBoxClient* client) const
@@ -589,7 +670,9 @@ void TabBoxHandler::createModel(bool partialReset)
         d->desktopModel()->createDesktopList();
         break;
     }
-    d->view->updateGeometry();
+    if (d->view) {
+        d->view->updateGeometry();
+    }
 }
 
 QModelIndex TabBoxHandler::first() const
